import logging
import os
import re
import textwrap
from enum import Enum
from json import JSONEncoder
from typing import Dict, Any, cast, Optional, List, Tuple, BinaryIO, Sequence

import requests
import urllib3
from requests import Response

from vectice.__version__ import __version__, __vectice_version__
from vectice.api.json_object import JsonObject

DEFAULT_API_ENDPOINT = "http://localhost:4000"

logger = logging.getLogger("vectice.http")


class HttpError(Exception):
    def __init__(self, code: int, reason: str, path: str, method: str, json: Optional[str]):
        super().__init__()
        self.code: int = code
        self.reason: str = reason
        self.path = path
        self.method = method
        self.json = json

    def __str__(self):
        begin = textwrap.dedent(
            f"""
            HTTP Error Code {self.code} : {self.reason}
            {self.method} {self.path}
        """
        )
        if self.json:
            end = textwrap.dedent(
                f"""
                ---- payload ---
                {self.json}
                ---
                """
            )
        else:
            end = ""
        return begin + end


def format_url(url: str) -> str:
    """Add https protocol if missing and remove trailing slash."""
    url = url.rstrip("/")
    if not re.match("(?:http|https|ftp)://", url):
        return "https://{}".format(url)
    return url


def log_request(method: str, path: str, headers: dict, payload: Optional[Any] = None) -> None:
    should_log = os.getenv("LOG_VECTICE_HTTP_REQUEST")
    if should_log is not None and should_log != "0" and should_log.lower() != "false":
        logger.info("###")
        logger.info(f"{method} {path}")
        for item in headers.items():
            logger.info(f"{item[0]}: {item[1] if item[0] != 'Authorization' else '********'}")
        logger.info(payload)
    else:
        logger.debug("###")
        logger.debug(f"{method} {path}")
        for item in headers.items():
            logger.debug(f"{item[0]}: {item[1] if item[0] != 'Authorization' else '********'}")
        logger.debug(payload)


def default_http_headers() -> Dict[str, str]:
    return {"Vectice-SDK-Version": __version__, "Vectice-Version": __vectice_version__}


class VecticeEncoder(JSONEncoder):
    """
    Json Encoder with 2 specific behaviors:
    - handle datetime types so be serialized as a string following ISO8601 format
    - remove any null property from the serialized json.
    - handle nested objects
    """

    def default(self, obj: Any) -> Any:
        from copy import deepcopy

        if hasattr(obj, "isoformat"):
            return obj.isoformat()
        if isinstance(obj, Enum):
            return obj.value
        internal_copy = deepcopy(obj.__dict__)
        return {k.lstrip("_"): v for (k, v) in internal_copy.items() if v is not None}


class Connection:
    def __init__(self, api_endpoint: Optional[str] = None, allow_self_certificate=True):
        self._API_BASE_URL = format_url(api_endpoint or DEFAULT_API_ENDPOINT)
        self._default_request_headers: Dict[str, str] = default_http_headers()
        self.verify_certificate = not allow_self_certificate
        if allow_self_certificate:
            urllib3.disable_warnings()

    @property
    def api_base_url(self) -> str:
        return self._API_BASE_URL

    def _clean_dict(self, payload: dict):
        cleaned_payload = {}
        for key, value in payload.items():
            if value is not None:
                if isinstance(value, Dict):
                    cleaned_payload[key] = self._clean_dict(value)
                else:
                    cleaned_payload[key] = value
        return cleaned_payload

    def _get(self, path: str) -> Dict[str, Any]:
        headers = {**self._default_request_headers, "Content-Type": "application/json"}
        log_request("GET", path, headers)
        response = requests.get(url=self.api_base_url + path, headers=headers, verify=self.verify_certificate)
        return self._response(self.api_base_url + path, headers, response, "GET")

    def _post(self, path: str, payload: Any = None) -> JsonObject:
        headers = {**self._default_request_headers, "Content-Type": "application/json"}
        cleaned_payload: Any = self._clean_dict(payload) if isinstance(payload, Dict) else payload
        data = VecticeEncoder(indent=1).encode(cleaned_payload)
        log_request("POST", path, headers, data)
        response = requests.post(
            url=self.api_base_url + path, headers=headers, data=data, verify=self.verify_certificate
        )
        return self._response(self.api_base_url + path, headers, response, "POST", data)

    def _put(self, path: str, payload: Any = None) -> JsonObject:
        headers = {**self._default_request_headers, "Content-Type": "application/json"}
        cleaned_payload: Any = self._clean_dict(payload) if isinstance(payload, Dict) else payload
        data = VecticeEncoder(indent=1).encode(cleaned_payload)
        log_request("PUT", path, headers, data)
        response = requests.put(
            url=self.api_base_url + path, headers=headers, data=data, verify=self.verify_certificate
        )
        return self._response(self.api_base_url + path, headers, response, "PUT", payload)

    def _delete(self, path: str, payload: Any = None) -> JsonObject:
        headers = {**self._default_request_headers, "Content-Type": "application/json"}
        if payload is None:
            data = None
        else:
            cleaned_payload: Any = self._clean_dict(payload) if isinstance(payload, Dict) else payload
            data = VecticeEncoder(indent=1).encode(cleaned_payload)
        log_request("DELETE", path, headers, data)
        response = requests.delete(
            url=self.api_base_url + path, headers=headers, data=data, verify=self.verify_certificate
        )
        return self._response(self.api_base_url + path, headers, response, "DELETE", data)

    def _post_attachments(
        self, path: str, files: Optional[Sequence[Tuple[str, Tuple[Any, BinaryIO]]]] = None
    ) -> Optional[Response]:
        headers = self._default_request_headers
        response = requests.post(
            url=self.api_base_url + path,
            headers=headers,
            files=files,  # type: ignore
            verify=self.verify_certificate,
        )
        return self._attachment_response(self.api_base_url + path, headers, response, "POST")

    def _put_attachments(
        self, path: str, files: Optional[List[Tuple[str, Tuple[Any, BinaryIO]]]] = None
    ) -> Optional[Response]:
        headers = self._default_request_headers
        response = requests.put(
            url=self.api_base_url + path,
            headers=headers,
            files=files,  # type: ignore
            verify=self.verify_certificate,
        )
        return self._attachment_response(self.api_base_url + path, headers, response, "PUT")

    def _get_attachment(self, path: str) -> Response:
        headers = self._default_request_headers
        response = requests.get(
            url=self.api_base_url + path, headers=headers, verify=self.verify_certificate, stream=True
        )
        return self._attachment_response(self.api_base_url + path, headers, response, "GET")

    def _delete_attachment(self, path: str) -> Optional[Response]:
        headers = self._default_request_headers
        response = requests.delete(url=self.api_base_url + path, headers=headers)
        return self._attachment_response(self.api_base_url + path, headers, response, "DELETE")

    def _list_attachments(self, path: str) -> Sequence[dict]:
        headers = {**self._default_request_headers, "Content-Type": "application/json"}
        response = requests.get(url=self.api_base_url + path, headers=headers, verify=self.verify_certificate)
        self._attachment_response(self.api_base_url + path, headers, response, "GET")
        return cast(Sequence[Dict], response.json())

    @classmethod
    def raise_status(cls, path: str, response: Response, method: str, payload: Optional[Any] = None) -> None:
        if not (200 <= response.status_code < 300):
            reason = response.text
            if not isinstance(payload, str):
                json = VecticeEncoder(indent=4, sort_keys=True).encode(payload) if payload is not None else None
            else:
                json = payload
            raise HttpError(response.status_code, reason, path, method, json)

    def _response(
        self, path: str, headers: Dict[str, str], response: Response, method: str, payload: Optional[Any] = None
    ) -> JsonObject:
        self.raise_status(path, response, method, payload)
        logger.debug(f"{method} {path} {response.status_code}")
        logger.debug("\n".join(f"{item[0]}: {item[1]}" for item in headers.items()))
        logger.debug(payload)
        if len(response.content) > 0:
            return cast(Dict[str, Any], response.json())
        return {response.reason: response.status_code}

    def _attachment_response(self, path: str, headers: Dict[str, str], response: Response, method: str) -> Response:
        self.raise_status(path, response, method)
        logger.debug(f"{method} {path} {response.status_code}")
        logger.debug("\n".join(f"{item[0]}: {item[1]}" for item in headers.items()))
        return response



from __future__ import annotations

import urllib
from typing import Optional, TYPE_CHECKING
from urllib.parse import urlencode

from .reference import InvalidReferenceError, MissingReferenceError, BadReferenceError
from ._auth import Auth
from ._http import HttpError
from .http_error_handlers import HttpErrorHandler
from .json import JobOutput, JobInput, Page, PagedResponse, JobType
from .project import ProjectApi

if TYPE_CHECKING:
    from vectice import Reference


class JobApi:
    def __init__(self, auth: Auth):
        self._auth = auth
        self._httpErrorhandler = HttpErrorHandler()

    def delete_job(self, job: Reference, project: Optional[Reference], workspace: Optional[Reference]) -> None:
        if isinstance(job, int):
            url = f"/metadata/job/{job}"
        elif isinstance(job, str):
            if project is not None:
                job_object = self.get_job(job, project, workspace)
                url = f"/metadata/job/{job_object.id}"
            else:
                raise MissingReferenceError("job", "project")
        else:
            raise InvalidReferenceError("job", job)
        try:
            self._auth._delete(url)
        except HttpError as e:
            raise self._httpErrorhandler.handleDeleteHttpError(e, "job", job)
        except IndexError:
            raise ValueError("The job is invalid. Please check the entered value.")

    def get_job(self, job: Reference, project: Optional[Reference], workspace: Optional[Reference]) -> JobOutput:
        parent_project = None
        if project and isinstance(job, int):
            parent_project = ProjectApi(self._auth).get_project(project, workspace)
            url = f"/metadata/project/{parent_project.id}/job/{job}"
        elif isinstance(job, int):
            url = f"/metadata/job/{job}"
        elif isinstance(job, str):
            if project is not None:
                parent_project = ProjectApi(self._auth).get_project(project, workspace)
                url = f"/metadata/project/{parent_project.id}/job/name/{urllib.parse.quote(job)}"
            else:
                raise MissingReferenceError("job", "project")
        else:
            raise InvalidReferenceError("job", job)
        try:
            response = self._auth._get(url)
            result = JobOutput(**response)
            if parent_project and result.project_id != parent_project.id:
                raise BadReferenceError("job", result.name, HttpError(0, "Not Found", url, "GET", ""))
            if result.project is None:
                result.project = parent_project
            return result
        except HttpError as e:
            raise self._httpErrorhandler.handleGetHttpError(e, "job", job)
        except IndexError:
            raise ValueError("The project is invalid. Please check the entered value.")

    def list_jobs(
        self,
        project: Reference,
        workspace: Optional[Reference] = None,
        search: str = None,
        page_index=Page.index,
        page_size=Page.size,
    ) -> PagedResponse[JobOutput]:
        if isinstance(project, str):
            parent_project = ProjectApi(self._auth).get_project(project, workspace)
            url = f"/metadata/project/{parent_project.id}/job"
        else:
            url = f"/metadata/project/{project}/job"
        queries = {"index": page_index, "size": page_size}
        if search:
            queries["search"] = search
        jobs = self._auth._get(url + "?" + urlencode(queries))
        return PagedResponse(item_cls=JobOutput, total=jobs["total"], page=jobs["page"], items=jobs["items"])

    def create_job(self, project: Reference, workspace: Optional[Reference], job: JobInput) -> JobOutput:
        if isinstance(project, str):
            parent_project = ProjectApi(self._auth).get_project(project, workspace)
            url = f"/metadata/project/{parent_project.id}/job"
        else:
            url = f"/metadata/project/{project}/job"
        if job.name is None:
            raise ValueError('"name" must be provided in job.')
        if job.type is None:
            job.type = JobType.OTHER
        return JobOutput(**self._auth._post(url, job.__dict__))

    def update_job(self, project: Optional[Reference], workspace: Optional[Reference], job: JobOutput) -> JobOutput:
        if isinstance(project, str):
            parent_project = ProjectApi(self._auth).get_project(project, workspace)
            url = f"/metadata/project/{parent_project.id}/job"
        else:
            url = f"/metadata/project/{project}/job"
        response = self._auth._put(url + "/" + str(job.id), job)
        return JobOutput(**response)



from dataclasses import dataclass
from typing import Optional, List


@dataclass
class User:
    """
    Defines the user entity inside Vectice
    """

    organizationId: int
    """
    """
    name: str
    """
    """
    email: str  # Do we need this in the SDK side
    """
    """
    password: str  # Do we need this in the SDK side
    """
    """
    token: str = ""
    """
    """
    workspaces: Optional[List[str]] = None
    """
    """
    requestId: Optional[str] = None
    """
    """
    inviteId: Optional[str] = None
    """
    """
    displayName: Optional[str] = None
    """
    """



from __future__ import annotations

import datetime
import logging
import urllib
from typing import Optional, TYPE_CHECKING, List
from urllib.parse import urlencode

from ._auth import Auth
from ._http import HttpError
from .http_error_handlers import HttpErrorHandler
from .job import JobApi
from .json import RunInput, RunOutput, Page, PagedResponse, PropertyOutput, PropertyInput
from .reference import InvalidReferenceError, BadReferenceError, MissingReferenceError

if TYPE_CHECKING:
    from vectice import Reference


class RunApi:
    def __init__(self, auth: Auth):
        self._auth = auth
        self._httpErrorHandler = HttpErrorHandler()
        self._logger = logging.getLogger(self.__class__.__name__)

    def list_runs(
        self,
        job: Reference,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
        search: str = None,
        page_index=Page.index,
        page_size=Page.size,
    ) -> PagedResponse[RunOutput]:
        if not isinstance(job, str) and not isinstance(job, int):
            raise InvalidReferenceError("job", job)
        parent_job = JobApi(self._auth).get_job(job, project, workspace)
        if parent_job.project is None:
            raise BadReferenceError("job", job)
        else:
            url = f"/metadata/project/{parent_job.project.id}/job/{parent_job.id}/run"
        queries = {"index": page_index, "size": page_size}
        if search:
            queries["search"] = search
        runs = self._auth._get(url + "?" + urlencode(queries))
        return PagedResponse(
            item_cls=RunOutput,
            total=runs["total"],
            page=runs["page"],
            items=runs["items"],
        )

    def get_run(
        self,
        run: Reference,
        job: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ) -> RunOutput:
        if isinstance(run, int):
            url = f"/metadata/run/{run}"
        elif isinstance(run, str):
            if job is None:
                raise MissingReferenceError("run", "job")
            parent_job = JobApi(self._auth).get_job(job, project, workspace)
            if parent_job is None:
                raise BadReferenceError("job")
            url = f"/metadata/project/{parent_job.project.id}/job/{parent_job.id}/run/name/{urllib.parse.quote(run)}"
        else:
            raise InvalidReferenceError("run", run)
        try:
            response = self._auth._get(url)
        except HttpError as e:
            raise self._httpErrorHandler.handleGetHttpError(e, "run", run)
        return RunOutput(**response)

    def create_run(
        self,
        data: RunInput,
        job: Reference,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ) -> RunOutput:
        if data.status is None:
            raise ValueError('"status" must be provided in run.')
        if data.startDate is None:
            raise ValueError('"startDate" must be provided in run.')
        parent_job = JobApi(self._auth).get_job(job, project, workspace)
        if data.name is None:
            data.name = parent_job.name + " " + datetime.datetime.now().isoformat()
        url = f"/metadata/project/{parent_job.project.id}/job/{parent_job.id}/run"
        try:
            response = self._auth._post(url, data)
            return RunOutput(**response)
        except HttpError as e:
            raise self._httpErrorHandler.handlePostHttpError(e, "run")

    def update_run(self, data: RunOutput) -> RunOutput:
        url = f"/metadata/run/{data.id}"
        response = self._auth._put(url, data)
        return RunOutput(**response)

    def delete_run(
        self,
        run: Reference,
        job: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ):
        if isinstance(run, int):
            url = f"/metadata/run/{run}"
        elif isinstance(run, str):
            run_output = self.get_run(run, job, project, workspace)
            url = f"/metadata/run/{run_output.id}"
        self._auth._delete(url)

    def create_run_properties(
        self,
        run: Reference,
        properties: List[PropertyInput],
        job: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ) -> RunOutput:
        parent_run = self.get_run(run, job, project, workspace)
        url = f"/metadata/project/{parent_run.job.project.id}/job/{parent_run.job.id}/run/{parent_run.id}"
        if properties:
            for prop in properties:
                self._auth._post(f"{url}/entityProperty/", prop)
        parent_run = self.get_run(run, job, project, workspace)
        self._logger.info(
            f"Properties with names: {[k.key for k in properties]} successfully added to Run {parent_run.name}."
        )
        return parent_run

    def list_run_properties(
        self,
        run: Reference,
        job: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
        page_index=Page.index,
        page_size=Page.size,
    ) -> PagedResponse[PropertyOutput]:
        parent_run = self.get_run(run, job, project, workspace)
        url = f"/metadata/project/{parent_run.job.project.id}/job/{parent_run.job.id}/run/{parent_run.id}"
        queries = {"index": page_index, "size": page_size}
        run_properties = self._auth._get(url + "/entityProperty?" + urlencode(queries))
        properties = [PropertyOutput.from_dict(property).as_dict() for property in run_properties["items"]]
        return PagedResponse(
            item_cls=PropertyOutput,
            total=run_properties["total"],
            page=run_properties["page"],
            items=properties,
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from ._http import HttpError
from .reference import BadReferenceError

if TYPE_CHECKING:
    from vectice import Reference


class HttpErrorHandler:
    def handleGetHttpError(self, e: HttpError, reference_type: str, reference: Reference) -> Exception:
        if e.code == 404:
            return BadReferenceError(reference_type, reference, e)
        elif e.code == 401:
            return PermissionError("bad or missing credentials")
        elif e.code == 403:
            return PermissionError(f"missing rights to access this {reference_type}")
        else:
            return RuntimeError(f"can not access {reference_type}: {e.reason}")

    def handlePostHttpError(self, e: HttpError, reference_type: str, action: str = "create") -> Exception:
        if e.code == 401:
            return PermissionError("bad or missing credentials")
        elif e.code == 403:
            return PermissionError(f"missing rights to access this {reference_type}")
        elif e.code == 400:
            return RuntimeError(f"can not {action} {reference_type}: {e.reason}")
        else:
            return RuntimeError(f"unexpected error: {e.reason}")

    def handlePutHttpError(self, e: HttpError, reference_type: str, reference: Reference) -> Exception:
        if e.code == 404:
            return BadReferenceError(reference_type, reference, e)
        elif e.code == 401:
            return PermissionError("bad or missing credentials")
        elif e.code == 403:
            return PermissionError(f"missing rights to access this {reference_type}")
        elif e.code == 400:
            return RuntimeError(f"can not update {reference_type} {reference}: {e.reason}")
        else:
            return RuntimeError(f"unexpected error: {e.reason}")

    def handleDeleteHttpError(self, e: HttpError, reference_type: str, reference: Reference) -> Exception:
        if e.code == 404:
            return BadReferenceError(reference_type, reference, e)
        elif e.code == 401:
            return PermissionError("bad or missing credentials")
        elif e.code == 403:
            return PermissionError(f"missing rights to access this {reference_type}")
        elif e.code == 400:
            return RuntimeError(f"can not delete {reference_type} {reference}: {e.reason}")
        else:
            return RuntimeError(f"unexpected error: {e.reason}")



from .client import Client
from .dataset import DatasetApi
from .dataset_version import DatasetVersionApi
from .job import JobApi
from .artifact import ArtifactApi
from .run import RunApi
from .model import ModelApi
from .model_version import ModelVersionApi
from .rule import RuleApi
from .code import CodeApi
from .code_version import CodeVersionApi
from .stage import StageApi

from .workspace import WorkspaceApi
from .reference import Reference, BadReferenceError, MissingReferenceError, InvalidReferenceError
from . import json

__all__ = [
    "BadReferenceError",
    "MissingReferenceError",
    "InvalidReferenceError",
    "Client",
    "CodeApi",
    "CodeVersionApi",
    "RuleApi",
    "RunApi",
    "JobApi",
    "RunApi",
    "ArtifactApi",
    "DatasetApi",
    "DatasetVersionApi",
    "ModelApi",
    "ModelVersionApi",
    "WorkspaceApi",
    "Reference",
    "json",
    "StageApi",
]



from __future__ import annotations

import urllib
from typing import Optional, TYPE_CHECKING, Dict, Any, List
from urllib.parse import urlencode

from ._auth import Auth
from ._http import HttpError
from ._utils import read_nodejs_date
from .dataset import DatasetApi
from .http_error_handlers import HttpErrorHandler
from .json import DatasetVersionOutput, DatasetVersionInput, Page, PagedResponse, FileMetadata
from .json.property import PropertyOutput
from .reference import MissingReferenceError

if TYPE_CHECKING:
    from vectice import Reference


class DatasetVersionApi:
    def __init__(self, auth: Auth):
        self._auth = auth
        self._httpErrorhandler = HttpErrorHandler()

    def get_dataset_version(
        self,
        version: Reference,
        dataset: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ) -> DatasetVersionOutput:
        if not isinstance(version, int) and not isinstance(version, str):
            raise ValueError("The dataset version reference is invalid. Please check the entered value.")
        if isinstance(version, int):
            url = f"/metadata/datasetversion/{version}"
        else:
            if dataset is None:
                raise MissingReferenceError("dataset version", "dataset")
            parent_dataset = DatasetApi(self._auth).get_dataset(dataset, project, workspace)
            url = f"/metadata/project/{parent_dataset.project.id}/dataset/{parent_dataset.id}/version/name/{urllib.parse.quote(version)}"
        try:
            response = self._auth._get(url)
            return DatasetVersionOutput(**response)
        except HttpError as e:
            raise self._httpErrorhandler.handleGetHttpError(e, "dataset version", version)
        except IndexError:
            raise ValueError("The dataset version is invalid. Please check the entered value.")

    def create_dataset_version(
        self,
        data: DatasetVersionInput,
        dataset: Reference,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ) -> DatasetVersionOutput:
        parent_dataset = DatasetApi(self._auth).get_dataset(dataset, project, workspace)
        try:
            data["datasetId"] = parent_dataset.id
            url = "/metadata/datasetversion"
            if "autoVersion" in data and data["autoVersion"]:
                url += "?autoVersion=true"
            response = self._auth._post(url, data)
            return DatasetVersionOutput(**response["version"], reusedVersion=response["reusedVersion"])
        except HttpError as e:
            raise self._httpErrorhandler.handlePostHttpError(e, "dataset version")
        except IndexError:
            raise ValueError("The dataset version is invalid. Please check the entered value.")

    def update_dataset_version(
        self,
        data: DatasetVersionOutput,
        version: Reference,
        dataset: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ) -> DatasetVersionOutput:
        dataset_version_object = self.get_dataset_version(version, dataset, project, workspace)
        url = f"/metadata/datasetversion/{dataset_version_object.id}"
        try:
            response = self._auth._put(url, data)
            return DatasetVersionOutput(**response)
        except HttpError as e:
            raise self._httpErrorhandler.handlePutHttpError(e, "dataset version", version)
        except IndexError:
            raise ValueError("The dataset version is invalid. Please check the entered value.")

    def list_dataset_versions(
        self,
        dataset: Reference,
        workspace: Optional[Reference] = None,
        project: Optional[Reference] = None,
        search: Optional[str] = None,
        page_index=Page.index,
        page_size=Page.size,
    ) -> PagedResponse[DatasetVersionOutput]:
        parent_dataset = DatasetApi(self._auth).get_dataset(dataset, project, workspace)
        url = f"/metadata/project/{parent_dataset.project.id}/dataset/{parent_dataset.id}/version"
        queries = {"index": page_index, "size": page_size}
        if search:
            queries["search"] = search
        dataset_versions = self._auth._get(url + "?" + urlencode(queries))
        return PagedResponse(
            item_cls=DatasetVersionOutput,
            total=dataset_versions["total"],
            page=dataset_versions["page"],
            items=dataset_versions["items"],
        )

    def delete_dataset_version(
        self,
        version: Reference,
        dataset: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ):
        try:
            dataset_version = self.get_dataset_version(version, dataset, project, workspace)
            self._auth._delete(f"/metadata/datasetversion/{dataset_version.id}")
        except HttpError as e:
            raise self._httpErrorhandler.handleGetHttpError(e, "dataset version", version)

    def list_files_metadata(
        self,
        version: Reference,
        dataset: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ) -> List[FileMetadata]:
        try:
            dataset_version = self.get_dataset_version(version, dataset, project, workspace)
            response = self._auth._get(
                f"/metadata/project/{dataset_version.dataset['project']['id']}/dataset/{dataset_version.dataset_id}/version/{dataset_version.id}/versionFolder"
            )
            return [
                FileMetadata(
                    name=item["name"],
                    id=item["id"],
                    path=item["path"]["path"],
                    type=item["type"],
                    isFolder=item["path"]["isFolder"],
                    children=[] if "children" not in item else item["children"],
                    size=int(item["size"]),
                    uri=item["uri"],
                    itemCreatedDate=read_nodejs_date(item["itemCreatedDate"]),
                    itemUpdatedDate=item["itemUpdatedDate"],
                )
                for item in response["files"]
            ]
        except HttpError as e:
            raise self._httpErrorhandler.handleGetHttpError(e, "version folder", "dataset version " + str(version))

    def create_dataset_version_properties(
        self,
        version: Reference,
        properties: Dict[str, Any],
        dataset: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ):
        dataset_version = self.get_dataset_version(version, dataset, project, workspace)
        url = f"/metadata/project/{dataset_version.dataset.project.id}/dataset/{dataset_version.dataset.id}/version/{dataset_version.id}"
        if isinstance(properties, dict):
            self._auth._post(f"{url}/entityProperty/", properties)
        parent_dataset_version = self.get_dataset_version(version, dataset, project, workspace)
        return parent_dataset_version

    def update_dataset_version_properties(
        self,
        version: Reference,
        property_id: int,
        properties: Dict[str, Any],
        dataset: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
    ):
        dataset_version = self.get_dataset_version(version, dataset, project, workspace)
        url = f"/metadata/project/{dataset_version.dataset.project.id}/dataset/{dataset_version.dataset.id}/version/{dataset_version.id}"
        self._auth._put(url + f"/entityProperty/{property_id}", properties)

    def list_dataset_version_properties(
        self,
        version: Reference,
        dataset: Optional[Reference] = None,
        project: Optional[Reference] = None,
        workspace: Optional[Reference] = None,
        page_index=Page.index,
        page_size=Page.size,
    ) -> PagedResponse[PropertyOutput]:
        dataset_version = self.get_dataset_version(version, dataset, project, workspace)
        url = f"/metadata/project/{dataset_version.dataset.project.id}/dataset/{dataset_version.dataset.id}/version/{dataset_version.id}"
        queries = {"index": page_index, "size": page_size}
        model_version_properties = self._auth._get(url + "/entityProperty?" + urlencode(queries))
        properties = [PropertyOutput.from_dict(property).as_dict() for property in model_version_properties["items"]]
        return PagedResponse(
            item_cls=PropertyOutput,
            total=model_version_properties["total"],
            page=model_version_properties["page"],
            items=properties,
        )



from __future__ import annotations

import logging
import sys
from typing import Optional

from vectice.api import Client
from vectice.models import GitVersion, ApiToken
from vectice.models.integration import AbstractIntegration


class IntegrationFactory:
    @staticmethod
    def create_adapter(
        lib: object,
        vectice_client: Client,
        *args,
        **kwargs,
    ) -> AbstractIntegration:
        if lib is not None:
            if "mlflow" in sys.modules:
                from mlflow.tracking.client import MlflowClient

                if isinstance(lib, MlflowClient):
                    from vectice.mlflow import MLflowIntegration

                    return MLflowIntegration(lib, vectice_client, *args, **kwargs)
        raise RuntimeError(f"There is no support library for {lib}")


class Integrations:
    """
    Factory class for Artifacts
    """

    @classmethod
    def create_code_version(cls, path: str = ".", check_remote_repository: bool = True) -> Optional[GitVersion]:
        """
        Create a code artifact based on the git information relative to the given local path.

        :param path: The path to look for the git repository
        :param check_remote_repository: if we should check remote repository for file existence
        :return: A CodeVersion or None if a git repository was not found locally
        """
        git_version = GitVersion.create(path, check_remote_repository)
        if git_version is not None:
            return git_version
        else:
            logging.warning(
                "Automatic code detection failed because the .git couldn't be found or file wasn't found by Github"
            )
            return None

    @classmethod
    def create_code_version_with_uri(
        cls, uri: str, script_relative_path: str, login_or_token=None, password=None, oauth2_token=None
    ) -> Optional[GitVersion]:
        if "github" in uri:
            code_version = Integrations.create_code_version_with_github_uri(
                uri, script_relative_path, login_or_token, password, oauth2_token
            )
        elif "bitbucket" in uri:
            code_version = Integrations.create_code_version_with_bitbucket_uri(
                uri, script_relative_path, login_or_token, password
            )
        else:
            code_version = Integrations.create_code_version_with_gitlab_uri(
                uri, script_relative_path, login_or_token, oauth2_token
            )
        if code_version is None:
            raise ValueError("Code Version was not created. Please check the provided uri and entrypoint.")
        return code_version

    @classmethod
    def create_code_version_with_github_uri(
        cls, uri: str, script_relative_path: str, login_or_token=None, password=None, oauth2_token=None
    ) -> Optional[GitVersion]:
        """
        Create a code artifact based on the github information relative to the given URI and relative path.

        Note: The URI given can include the branch you are working on. otherwise, the default repository branch will be used.

        sample :
            https://github.com/my-organization/my-repository (no branch given so using default branch)
            https://github.com/my-organization/my-repository/tree/my-current-branch (branch given is my-current-branch)

        To access private repositories, you need to authenticate with your credentials.
        see https://docs.github.com/en/github/authenticating-to-github/keeping-your-account-and-data-secure/about-authentication-to-github

        :param uri: The uri of the repository with a specific branch if needed.
        :param script_relative_path:  The file that is executed
        :param login_or_token: A real login or personal access token
        :param password: The password
        :param oauth2_token: The Oauth2 access token
        :return: A CodeVersion or None if the github repository was not found or is not accessible
        """
        git_version = GitVersion.create_from_github_uri(
            uri, script_relative_path, login_or_token, password, oauth2_token
        )
        return git_version

    @classmethod
    def create_code_version_with_gitlab_uri(
        cls,
        uri: str,
        script_relative_path: str,
        private_token: Optional[str] = None,
        oauth2_token: Optional[str] = None,
    ) -> Optional[GitVersion]:
        """
        Create an artifact that contains a version of a code

        :param uri: The uri of the repository with a specific branch if needed.
        :param script_relative_path:  The file that is executed
        :param private_token: A real login or a personal access token
        :param oauth2_token: The OAuth2 access token
        :return: A CodeVersion or None if the GitHub repository was not found or is not accessible
        """
        git_version = GitVersion.create_from_gitlab_uri(uri, script_relative_path, private_token, oauth2_token)
        return git_version

    @classmethod
    def create_code_version_with_bitbucket_uri(
        cls,
        uri: str,
        script_relative_path: str,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ) -> Optional[GitVersion]:
        """
        Create a code artifact based on the Bitbucket information relative to the given URI and relative path.

        Note: The URI given can include the branch you are working on. otherwise, the default repository branch will be used.

        sample :
            https://bitbucket.org/workspace/project/ (no branch given so using default branch)
            https://bitbucket.org/workspace/project/src/branch (branch given is my-current-branch)

        To access private repositories, you need to authenticate with your credentials.
        see Bitbucket Cloud: https://atlassian-python-api.readthedocs.io/index.html

        :param uri: The uri of the repository with a specific branch if needed.
        :param script_relative_path:  The file that is executed
        :param username: Bitbucket email
        :param password: Bitbucket password
        :return: A CodeVersion or None if the Bitbucket repository was not found or is not accessible
        """
        git_version = GitVersion.create_from_bitbucket_uri(uri, script_relative_path, username, password)
        return git_version

    @classmethod
    def parse_api_token(cls, token_file: str) -> ApiToken:
        """
        Parses the the API Token json file and sets the os environmental variable for the "VECTICE_API_TOKEN"
        for the user.

        :param token_file: The filepath to the json file containing the "VECTICE_API_TOKEN", found on the Vectice App
        :return: ApiToken or None if the json file can not be parsed or found
        """
        return ApiToken.parse_api_token_json(token_file)



from typing import Optional
from .project import ProjectOutput
from enum import Enum


class StageOrigin(Enum):
    """
    Indicates the Documentation Page Origin.
    """

    VecticeFile = "VecticeFile"
    """
    """
    ExternalLink = "ExternalLink"
    """
    """


class StageStatus(Enum):
    """
    Indicates the Documentation Page Status.
    """

    NotStarted = "NotStarted"
    """
    """
    InProgress = "Draft"
    """
    """
    Completed = "Completed"
    """
    """


class DocumentationPageType(Enum):
    """
    Indicates the Documentation Page Type.
    """

    Project = "Project"
    """
    """
    Dataset = "Dataset"
    """
    """
    Model = "Model"
    """
    """


class StageInput(dict):
    @property
    def name(self) -> str:
        return str(self["name"])

    @property
    def status(self) -> Optional[str]:
        return str(self["status"])

    @property
    def uri(self) -> str:
        return str(self["uri"])


class StageOutput(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if "project" in self:
            self._project: ProjectOutput = ProjectOutput(**self["project"])
        else:
            self._project = None

    def items(self):
        result = []
        for key in self:
            if self[key] is not None:
                result.append((key, self[key]))
        return result

    @property
    def id(self) -> int:
        return int(self["id"])

    @property
    def name(self) -> str:
        return str(self["name"])

    @property
    def status(self) -> str:
        return str(self["status"])

    @property
    def type(self) -> str:
        return str(self["type"])

    @property
    def origin(self) -> str:
        return str(self["origin"])

    @property
    def index(self) -> int:
        return int(self["index"])

    @property
    def uri(self) -> str:
        return str(self["uri"])

    @property
    def create_date(self) -> str:
        return str(self["createdDate"])

    @property
    def updated_date(self) -> str:
        return str(self["updatedDate"])

    @property
    def deleted_date(self) -> str:
        return str(self["deletedDate"])

    @property
    def project_id(self) -> int:
        return int(self["projectId"])

    @property
    def project(self) -> ProjectOutput:
        return self._project

    @project.setter
    def project(self, project: ProjectOutput):
        self._project = project



from __future__ import annotations

from typing import Optional, Dict


from vectice.api.json.block import BlockType, BlockState, BlockVariant


class Block:
    """
    Describes a block that belongs to page
    """

    def __init__(
        self,
        type: BlockType,
        variant: BlockVariant,
        state: Optional[BlockState] = None,
        name: Optional[str] = None,
        position: Optional[int] = None,
        content: Optional[str] = None,
        resource_id: Optional[int] = None,
        entity_file_id: Optional[int] = None,
        file_path: Optional[str] = None,
        file_size: Optional[int] = None,
        url: Optional[str] = None,
        config: Optional[Dict] = None,
    ):
        """
        :param type: the block type
        :param variant: the block variant
        :param state: the block state
        :param name: the name of the block
        :param position: the position of the block
        :param content: the block content such as text
        :param resource_id: the vectice resource id
        :param entity_file_id: the entity file id
        :param file_path: the file path of the notebook/image
        :param file_size: the size of the file
        :param url: the url of the block


        """
        self._type = type
        self._variant = variant
        self._state = state
        self._name = name
        self._position = position
        self._content = content
        self._resource_id = resource_id
        self._entity_file_id = entity_file_id
        self._file_path = file_path
        self._file_size = file_size
        self._url = url
        self._config = config

    def __repr__(self):
        return f"Block(position={self.position}, type={self.type}, variant={self.variant}, resource_id={self.resource_id}, content={self.content})"

    @property
    def type(self) -> BlockType:
        return self._type

    @property
    def variant(self) -> BlockVariant:
        return self._variant

    @property
    def content(self) -> Optional[str]:
        return self._content

    @property
    def name(self) -> Optional[str]:
        return self._name

    @property
    def position(self) -> Optional[int]:
        return self._position

    @property
    def searchable_content(self) -> Optional[str]:
        return self._content

    @property
    def state(self) -> Optional[BlockState]:
        return self._state

    @property
    def resource_id(self) -> Optional[int]:
        return self._resource_id

    @property
    def entity_file_id(self) -> Optional[int]:
        return self._entity_file_id

    @property
    def file_size(self) -> Optional[int]:
        return self._file_size

    @property
    def file_path(self) -> Optional[str]:
        return self._file_path

    @property
    def url(self) -> Optional[str]:
        return self._url

    @property
    def config(self) -> Optional[Dict]:
        return self._config



from datetime import datetime
from enum import Enum
from typing import Optional

from .project import ProjectOutput
from .._utils import read_nodejs_date


class ModelType(Enum):
    """
    Indicates the Model Type.
    """

    ANOMALY_DETECTION = "ANOMALY_DETECTION"
    """
    """
    CLASSIFICATION = "CLASSIFICATION"
    """
    """
    CLUSTERING = "CLUSTERING"
    """
    """
    OTHER = "OTHER"
    """
    """
    RECOMMENDATION_MODELS = "RECOMMENDATION_MODELS"
    """
    """
    REGRESSION = "REGRESSION"
    """
    """
    TIME_SERIES = "TIME_SERIES"
    """
    """


class ModelInput(dict):
    def items(self):
        result = []
        for key in self:
            if self[key] is not None:
                result.append((key, self[key]))
        return result

    @property
    def name(self) -> str:
        return str(self["name"])

    @property
    def type(self) -> ModelType:
        return ModelType(self["type"])

    @property
    def description(self) -> str:
        return str(self["description"])

    @property
    def repository(self) -> str:
        return str(self["repository"])


class ModelOutput(dict):
    _project: ProjectOutput

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if "project" in self:
            self._project = ProjectOutput(**self["project"])
        else:
            self._project = None

    def items(self):
        result = []
        for key in self:
            if self[key] is not None:
                result.append((key, self[key]))
        return result

    @property
    def name(self) -> str:
        return str(self["name"])

    @property
    def type(self) -> str:
        return str(self["type"])

    @property
    def description(self) -> Optional[str]:
        if "description" in self and self["description"] is not None:
            return str(self["description"])
        else:
            return None

    @property
    def created_date(self) -> Optional[datetime]:
        return read_nodejs_date(str(self["createdDate"]))

    @property
    def updated_date(self) -> Optional[datetime]:
        return read_nodejs_date(str(self["updatedDate"]))

    @property
    def version(self) -> int:
        return int(self["version"])

    @property
    def id(self) -> int:
        return int(self["id"])

    @property
    def author_id(self) -> int:
        return int(self["authorId"])

    @property
    def project_id(self) -> int:
        return int(self["projectId"])

    @property
    def repository(self) -> str:
        return str(self["repository"])

    @property
    def deleted_date(self) -> Optional[datetime]:
        return read_nodejs_date(str(self["deletedDate"]))

    @property
    def project(self) -> ProjectOutput:
        return self._project

    @project.setter
    def project(self, project: ProjectOutput):
        self._project = project



from __future__ import annotations

import logging
from typing import Optional, TYPE_CHECKING, List, Union


from vectice.api import Client
from vectice.api.json.stage import StageStatus, StageOrigin
from vectice.api.json.block import BlockInput, BlockType, BlockVariant, BlockState
from .block import Block
from .attachment_container import AttachmentContainer

if TYPE_CHECKING:
    from vectice.models import Project


class Stage:
    """
    Describes a Documentation Page
    """

    def __init__(
        self,
        id: int,
        project: Project,
        name: str,
        status: Union[StageStatus, str] = StageStatus.InProgress,
        origin: Optional[StageOrigin] = None,
    ):
        """
        :param id: the documentation page identifier
        :param project: the project the dataset belong to
        :param name: the name of the documentation page
        :param status: the name of the documentation status

        """
        self._id = id
        self._name = name
        self._status = status
        self._origin = origin
        self._project: Project = project
        self._client: Client = project._client
        self._logger = logging.getLogger(self.__class__.__name__)

    def __repr__(self):
        return f"Stage(id={self.id}, name={self.name}, status={self.status}, origin={self.origin})"

    @property
    def id(self) -> int:
        """
        Stage identifier.
        :return: int
        """
        return self._id

    @id.setter
    def id(self, dataset_id: int):
        self._id = dataset_id

    @property
    def project(self) -> Project:
        """
        The project this stage belong to.
        :return: Optional[Project]
        """
        return self._project

    @property
    def name(self) -> str:
        """
        Name of the stage
        :return: str
        """
        return self._name

    @property
    def status(self) -> Union[StageStatus, str]:
        """
        Name of the stage
        :return: str
        """
        return self._status

    @property
    def origin(self) -> Optional[StageOrigin]:
        """
        Origin of the stage
        :return: str
        """
        return self._origin

    def add_block(
        self,
        dataset: Optional[int] = None,
        dataset_version: Optional[int] = None,
        model: Optional[int] = None,
        model_version: Optional[int] = None,
        run: Optional[int] = None,
        text: Optional[str] = None,
        formula: Optional[str] = None,
        code: Optional[str] = None,
        divider: Optional[bool] = False,
        path: Optional[str] = None,
        position: Optional[int] = None,
    ) -> Stage:
        """
        Add a block to the documentation stage/documentation which belongs to a Dataset or Model. Project refers to a stage and Dataset or Model refers to the documentation for the Dataset or Model. If no position is provided then the block is added at the end of the documentation.

        If the artifact version name is provided as a string then the artifact reference is needed to get the artifact version.

        :param dataset: The dataset id
        :param dataset_version: The model id
        :param model: The model id
        :param model_version: The model id
        :param run: The run id
        :param text: The text that will populate the block.
        :param formula: The formula of the block
        :param code: The code of the block
        :param divider: Add a divider
        :param path: The path to the NOTEBOOK/FILE/IMAGE
        :param position: The position to add the block.
        :return: None
        """
        if dataset or dataset_version or model or model_version or run:
            block_type = BlockType.VECTICE_RESOURCE
            if dataset:
                variant = BlockVariant.DATASET
                resource_id = dataset
                name = self._client.get_dataset(dataset).name
            elif dataset_version:
                variant = BlockVariant.DATASET_VERSION
                resource_id = dataset_version
                name = self._client.get_dataset_version(dataset_version).name
            elif model:
                variant = BlockVariant.MODEL
                resource_id = model
                name = self._client.get_model(model).name
            elif model_version:
                variant = BlockVariant.MODEL_VERSION
                resource_id = model_version
                name = self._client.get_model_version(model_version).name
            elif run:
                variant = BlockVariant.RUN
                resource_id = run
                name = self._client.get_run(run).name
            else:
                raise ValueError("Please provide valid input for the block.")
            block_input = BlockInput(
                variant=variant,
                type=block_type,
                resourceId=resource_id,
                position=position,
                name=name,
                state=BlockState.FULL,
            )
        elif text:
            block_type = BlockType.EDITABLE
            variant = BlockVariant.TEXT
            block_input = BlockInput(
                variant=variant, type=block_type, content=f"<p>{text}</p>", searchableContent=text, position=position
            )
        elif divider:
            block_type = BlockType.OTHER
            variant = BlockVariant.DIVIDER
            block_input = BlockInput(
                variant=variant, type=block_type, content=text, searchableContent=text, position=position
            )
        elif code:
            block_type = BlockType.EDITABLE
            variant = BlockVariant.CODE
            block_input = BlockInput(
                variant=variant, type=block_type, content=code, searchableContent=code, position=position
            )
        elif formula:
            block_type = BlockType.EDITABLE
            variant = BlockVariant.FORMULA
            block_input = BlockInput(
                variant=variant, type=block_type, content=formula, searchableContent=formula, position=position
            )
        elif path:
            attachment_container = AttachmentContainer(self.name, self.id, self.project._client, "project")
            try:
                attachment_container.add_attachments(file_path=path)
                attachments = attachment_container._list_attachments()
                if attachments[-1].fileName == path.split("/")[-1]:
                    file_name = attachments[-1].fileName
                    file_type = attachments[-1].contentType
                    file_id = attachments[-1].fileId
                else:
                    raise ValueError("Please check the path provided.")
            except RuntimeError:
                self._logger.warning(
                    f"The file '{path}' already exists, update the file in Vectice if this version is newer."
                )
                attachments = attachment_container._list_attachments()
                file_name, file_type, file_id = None, None, None
                for attachment in attachments:
                    if attachment.fileName == path.split("/")[-1]:
                        file_name = attachment.fileName
                        file_type = attachment.contentType
                        file_id = attachment.fileId
                if file_name is None or file_type is None or file_id is None:
                    raise ValueError("File could not be found in Vectice.")
            block_type = BlockType.CLOUD_FILE
            if file_type == "Notebook":
                variant = BlockVariant.NOTEBOOK
            elif file_type == "ImageFile":
                variant = BlockVariant.IMAGE
            elif file_type is not None and isinstance(file_type, str):
                variant = BlockVariant.FILE
            else:
                raise ValueError(f"Unknown file type {file_type}")
            block_input = BlockInput(
                variant=variant,
                type=block_type,
                position=position,
                entityFileId=file_id,
                name=file_name,
                state=BlockState.FULL,
            )
        else:
            raise ValueError("Please provide valid input for the block.")
        result = self._client.create_block(self.id, block_input, self.project.id, self.project.workspace.id)
        self._logger.info(
            f"Block(type={result.type}, variant={result.variant}, position={result.position}) has been created in the Stage '{self.name}'"
        )
        return self

    def _add_header(self, text: str) -> Stage:
        block_type = BlockType.EDITABLE
        variant = BlockVariant.TEXT
        block_input = BlockInput(
            variant=variant, type=block_type, content=f"<h2><strong>{text}</strong></h2>", searchableContent=text
        )
        result = self._client.create_block(self.id, block_input)
        self._logger.info(
            f"Block(type={result.type}, variant={result.variant}, position={result.position}) has been created in the Stage '{self.name}'"
        )
        return self

    def _add_model_table(self, model_id: Union[int, str], model_version_id: int) -> Stage:
        block_type = BlockType.VECTICE_RESOURCE
        variant = BlockVariant.MODEL
        model = self._client.get_model(model_id)
        resource_id = model.id
        model_version = self._client.get_model_version(model_version_id)
        block_input = BlockInput(
            variant=variant,
            type=block_type,
            resourceId=resource_id,
            name=model.name,
            state=BlockState.FULL,
            config={"filters": '{"name":[{"key":"name","match":"EQUAL","value":"%s"}]}' % model_version.name},
        )
        self._client.create_block(self.id, block_input)
        return self

    def _add_attachment(self, attachment) -> Stage:
        file_name = attachment.fileName
        file_type = attachment.contentType
        file_id = attachment.fileId
        block_type = BlockType.CLOUD_FILE
        if file_type == "Notebook":
            variant = BlockVariant.NOTEBOOK
        elif file_type == "ImageFile":
            variant = BlockVariant.IMAGE
        elif file_type is not None and isinstance(file_type, str):
            variant = BlockVariant.FILE
        else:
            raise ValueError(f"Unknown file type {file_type}")
        block_input = BlockInput(
            variant=variant,
            type=block_type,
            entityFileId=file_id,
            name=file_name,
            state=BlockState.FULL,
        )
        self._client.create_block(self.id, block_input)
        return self

    def delete_block(self, position: int) -> None:
        """
        Deletes a file block.

        :param position: The position of the block.

        :return: Stage
        """
        self._client.delete_block(position, self.id, self.project.id, self.project.workspace.id)
        self._logger.info(f"Block at position {position} successfully deleted.")

    def list_blocks(self) -> List[Block]:
        """
        List the blocks associated to a stage.

        :return: List of Blocks
        """
        result = self._client.list_blocks(self.id, self.project.id, self.project.workspace.id)
        self._logger.info("Blocks successfully retrieved.")
        return [
            Block(
                position=item.position,
                type=BlockType[item.type],
                variant=BlockVariant[item.variant],
                state=BlockState[item.state] if item.state is not None else None,
                content=item.content,
                name=item.name,
                resource_id=item.resource_id,
                entity_file_id=item.entity_file_id,
            )
            for item in result
        ]



from dataclasses import dataclass
from datetime import datetime

from typing import Optional

from vectice.api._utils import read_nodejs_date


@dataclass
class WorkspaceInput:
    name: Optional[str]
    description: Optional[str]


class WorkspaceOutput(dict):
    def items(self):
        result = []
        for key in self:
            if self[key] is not None:
                result.append((key, self[key]))
        return result

    @property
    def organization_id(self) -> int:
        return int(self["organizationId"])

    @property
    def id(self) -> int:
        return int(self["id"])

    @property
    def name(self) -> str:
        return str(self["name"])

    @property
    def description(self) -> Optional[str]:
        if "description" in self and self["description"] is not None:
            return str(self["description"])
        else:
            return None

    @property
    def created_date(self) -> Optional[datetime]:
        return read_nodejs_date(str(self["createdDate"]))

    @property
    def updated_date(self) -> Optional[datetime]:
        return read_nodejs_date(str(self["updatedDate"]))

    @property
    def deleted_date(self) -> Optional[datetime]:
        return read_nodejs_date(str(self["deletedDate"]))

    @property
    def version(self) -> int:
        return int(self["version"])
